/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hli@hendriklipka.de
*/
package de.hendriklipka.iwg;

import de.hendriklipka.iwg.standalone.JavaFileCreator;
import de.hendriklipka.iwg.data.ClassDescriptor;
import de.hendriklipka.iwg.body.PlainBodygenerator;

import gnu.getopt.*;

import java.lang.reflect.*;
import java.util.*;

/**
 * @todo handle variable tabulator sizes
 * @todo handle tab character instead of spaces as tab
 * @todo handle command line parameters
 * @todo better commenting of generated code
 * @todo handle adding code to existing gclasses
 */
public class Implementor
{
    protected IJavaCreator _creator=null;
    protected IBodyGenerator _bodyGen=null;

    protected String[] _interfaces=null;
    protected boolean _implementProtected=false;

    protected ClassDescriptor _classDesc=null;

    public Implementor(String[] interfaces, String className, IJavaCreator creator, IBodyGenerator bodyGen, boolean implementProtected)
    {
        _creator=creator;
        _bodyGen=bodyGen;

        _classDesc=new ClassDescriptor();
        String packageName=null;
        int pos=className.lastIndexOf('.');
        if (-1!=pos)
        {
            packageName=className.substring(0,pos);
            className=className.substring(pos+1);
        }
        _classDesc._package=packageName;
        _classDesc._name=className;
//        _classDesc._interfaces=interfaces;
        _interfaces=interfaces;
        _implementProtected=implementProtected;
    }

    public void createClassDefinition()
    {
        InterfaceScanner is=new InterfaceScanner(_bodyGen);
        for (int i=0;i<_interfaces.length;i++)
        {
            try
            {
                is.addInterfaceToClass(_interfaces[i],_classDesc,_implementProtected);
            }
            catch (ClassNotFoundException e)
            {
            }
        }
    }

    public void implementClass()
    {
        _classDesc.shortenTypeNames();
        _creator.create(_classDesc);
    }

    public static void main(String[] params)
    {
        LongOpt[] opts=new LongOpt[3];
        opts[0]=new LongOpt("help",LongOpt.NO_ARGUMENT,null,'h');
        opts[1]=new LongOpt("classname",LongOpt.REQUIRED_ARGUMENT,null,'c');
        opts[2]=new LongOpt("sourcepath",LongOpt.REQUIRED_ARGUMENT,null,'s');
        Getopt options=new Getopt("java "+Implementor.class.getName(),params,"",opts,true);

        Vector interfaces=new Vector();
        String sourcePath=".";
        String classname=null;
        int c;
        boolean error=false;
        while ((c = options.getopt()) != -1)
        {
            switch (c)
            {
                case 'c':
                    classname=options .getOptarg();
                    break;
                case 's':
                    sourcePath=options.getOptarg();
                    break;
                case 'h':
                default:
                    error=true;
                    break;
            }
        }
        if (error)
        {
            printHelp();
            return;
        }
        if (null==classname)
        {
            System.err.println("ERROR: classname not specified!");
            printHelp();
            return;
        }

        for (int i = options.getOptind(); i < params.length ; i++)
        {
            interfaces.addElement(params[i]);
        }
        String[] ifs=new String[interfaces.size()];
        interfaces.copyInto(ifs);
        IBodyGenerator bg=new PlainBodygenerator();
        JavaFileCreator jfc=new JavaFileCreator(sourcePath,bg);
        Implementor impl=new Implementor(ifs,classname,jfc,bg,false);
        impl.createClassDefinition();
        impl.implementClass();
    }

    private static void printHelp()
    {
        System.err.println("java "+Implementor.class.getName()+" --classname=CLASSNAME --sourcepath=PATH interfaces");
    }
}