/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
hli@hendriklipka.de
*/
package de.hendriklipka.iwg.body;

import de.hendriklipka.iwg.IBodyGenerator;
import de.hendriklipka.iwg.data.MethodDescriptor;
import de.hendriklipka.iwg.data.ParameterDescriptor;
import de.hendriklipka.iwg.data.ClassDescriptor;

public class PlainBodygenerator implements IBodyGenerator
{
    public String[] generateConstructorInterface(ClassDescriptor classDesc)
    {
        String[] sa=new String[1];
        StringBuffer line=new StringBuffer("public ");
        line.append(classDesc._name);
        line.append("(");
        for (int i=0; i<classDesc._fields.length; i++)
        {
            ParameterDescriptor field=classDesc._fields[i];
            String pName=createParameterName(field._type);
            line.append(field._type);
            line.append(" ");
            line.append(pName);
            if (i!=(classDesc._fields.length-1))
                line.append(", ");
        }
        line.append(")");
        sa[0]=line.toString();
        return sa;
    }
    
    public String[] generateConstructorBody(ClassDescriptor classDesc)
    {
        String[] sa=new String[classDesc._fields.length];
        for (int i=0; i<classDesc._fields.length; i++)
        {
            StringBuffer line=new StringBuffer();
            ParameterDescriptor field=classDesc._fields[i];
            line.append(field._name);
            line.append("=");
            line.append(createParameterName(field._type));
            line.append(";");
            sa[i]=line.toString();
        }
        return sa;
    }
    
    public String[] generateMethodInterface(MethodDescriptor method)
    {
        String[] methodInterface=new String[2];
        StringBuffer line=new StringBuffer();
        line.append("// method implemented from interface ");
        line.append(method._origin);
        methodInterface[0]=line.toString();

        line=new StringBuffer();
        if (method._isPublic)
            line.append("public");
        else
            line.append("protected");
        line.append(" ");
        if (method._isSynchronized)
            line.append("synchronized ");
        line.append(method._returnType);
        line.append(" ");
        line.append(method._name);
        line.append("(");
        if ((null!=method._params)&&(0!=method._params.length))
        {
            for (int i = 0; i < method._params.length; i++)
            {
                ParameterDescriptor param = method._params[i];
                line.append(param._type);
                line.append(" ");
                line.append(param._name);
                if (i!=(method._params.length-1))
                    line.append(", ");
            }
        }
        line.append(")");
        if ((null!=method._exceptions)&&(0!=method._exceptions.length))
        {
            line.append(" throws ");
            for (int i = 0; i < method._exceptions.length; i++)
            {
                String exception = method._exceptions[i];
                line.append(exception);
                if (i!=(method._exceptions.length-1))
                    line.append(", ");
            }
        }
        methodInterface[1]=line.toString();
        return methodInterface;
    }

    public String[] generateMethodBody(MethodDescriptor method)
    {
//        System.out.println("generating body for ");
        String[] methodBody=new String[1];
        StringBuffer line=new StringBuffer();
        if (!method._returnType.equals("void"))
            line.append("return ");
        line.append(method._fieldName);
        line.append(".");
        line.append(method._name);
        line.append("(");
        for (int i = 0; i < method._params.length; i++)
        {
            ParameterDescriptor param = method._params[i];
            line.append(param._name);
            if (i!=(method._params.length-1))
                line.append(",");
        }
        line.append(");");
        methodBody[0]=line.toString();
        return methodBody;
    }

    public String createFieldName(String type)
    {
        String fName=getShortTypeName(type);
        fName="_orig"+fName;
        return fName;
    }

    public String createParameterName(String type)
    {
        String fName=getShortTypeName(type);
        fName="a"+fName;
        return fName;
    }

    private String getShortTypeName(String type)
    {
        String fName=type;
        int pos=type.lastIndexOf('.');
        if (-1!=pos)
        {
            fName=type.substring(pos+1);
        }
        return fName;
    }

    public String[] createHeader()
    {
        String[] header={"// created by InterfaceWrapperGenerator","// (C) 2001,2002 by Hendrik Lipka"};
        return header;
    }

    public String[] createFooter()
    {
        String[] footer={"// end of generated class"};
        return footer;
    }
}
